###Script used in the analysis of the neuron data from section 4.2.1 of Penalized Estimation in Large-Scale Generalized Linear Array Models.

### Adam Lund 2016

library(glmnet)
library(glamlasso)
library(Matrix)
sparse <- 1
runglmnet <- 1

####################### Load data
load("1GB/Bs.rda")
load("1GB/Br.rda")
load("1GB/Bc.rda")
load("1GB/Y.rda")
Ytr <- Y 
 
####################### Arrange data 
Y <- aperm(Ytr, c(3, 2 ,1))   
X1 <- Bs
X2 <- Bc
X3 <- Br

sX1 <- Matrix(X1, sparse = TRUE)
sX2 <- Matrix(X2, sparse = TRUE)
sX3 <- Matrix(X3, sparse = TRUE)

prod(dim(X1)) * prod(dim(X2)) * prod(dim(X3)) / 1024^3 * 8

####################### Make the design
if(runglmnet == 1){

if(sparse == 1){

system.time(X <- kronecker(sX3, kronecker(sX2, sX1)))

}else{

system.time(X <- kronecker(X3, kronecker(X2, X1)))

}

object.size(X) / 1024^3

}

####################### Time trials
########### glmnet 
if(runglmnet == 1){
	
system.time(
fitglmnet <- glmnet(X, 
                    c(Y), 
                    family = "gaussian", 
                    standardize = FALSE, 
                    intercept = FALSE, 
                    thresh = 1e-01) 
           )

}

if(runglmnet != 1){

lambda <- NULL
nlambda <- 100

}else{

lambda <- fitglmnet$lambda
nlambda <- length(lambda)

}

########### glamlasso 
system.time(
fitglamlasso <- glamlasso(list(X1, X2, X3),
                          Y, 
                          family = "gaussian",
                          penalty = "lasso",
                          nlambda = length(fitglmnet$lambda),
                          lambda.min.ratio = 1e-04,
                          lambda = lambda,
                          penalty.factor = NULL,
                          reltolinner = 5 * 1e-07, 
                          reltolouter = 1e-04, 
                          maxiter = 25000,
                          maxiterinner = 3000, 
                          maxiterouter = 25,
                          iwls = "exact",
                          nu = .01)
)

####################### Plot objectives
objglmnet <- objective(Y, Weights = NULL,  list(X1, X2, X3), fitglmnet$beta, fitglmnet$lambda, penalty.factor = NULL, family = "gaussian", penalty = "lasso") 
objglamlasso <- objective(Y, Weights = NULL, list(X1, X2, X3), fitglamlasso$coef, fitglamlasso$lambda, penalty.factor = NULL, family = "gaussian", penalty = "lasso") 

m <- min(objglamlasso, objglmnet)
M <- max(objglamlasso, objglmnet)
par(mfrow = c(2, 1))
plot(objglmnet, type = "l", ylim = c(m, M), xlab = "model no.", main = "objglmnet (black) and objglamlasso (red)")
lines(objglamlasso, col = "red")
ratio <- (objglamlasso - objglmnet) / abs(objglmnet)
plot(ratio, type = "l", ylim = c(min(0, ratio, na.rm = TRUE), max(0, ratio, na.rm=TRUE)), col = "red", xlab = "model no.", main = "(objglamlasso - objglmnet) / abs(objglmnet) (red)")
abline(h = 0)
