###Script used in the simulation study in section 6.5 of Penalized Estimation in Large-Scale Generalized Linear Array Models.

### Adam Lund 2016

library(Matrix)
library(glmnet)
library(glamlasso)
library(MASS)

mult <- 0.7  #0.7,0.76,0.9,0.95,1.0,1.05,1.06,1.12
runglmnet <- 1

reps <- 5
seed <- sample(1:10^6, reps) 
Objglm <- Objglam  <- matrix(NA, reps, 100)
glmtime <- glamtime <- rep(NA, reps)

for (i in 1:reps){
	
######################### Size of example
n1 <- ceiling(60 * mult) #biggest!!
n2 <- ceiling(20 * mult)
n3 <- ceiling(10 * mult)
n <- n1 * n2 * n3
frac <- 0.5
p1 <- max(3, ceiling(n1 / frac))
p2 <- max(3, ceiling(n2 / frac))
p3 <- max(3, ceiling(n3 / frac))
p <- p1 * p2 * p3

######################### Simulate covariates
set.seed(seed[i])
rho <- 0
sigma <- 1
	
Sigma1 <- matrix(rho, p1, p1)
diag(Sigma1) <- rep(sigma, p1)
Sigma2 <- matrix(rho, p2, p2)
diag(Sigma2) <- rep(sigma, p2)
Sigma3 <- matrix(rho, p3, p3)
diag(Sigma3) <- rep(sigma, p3)

X1tr <- mvrnorm(n1, rep(0, p1), Sigma1) 
X2tr <- mvrnorm(n2, rep(0, p2), Sigma2) 
X3tr <- mvrnorm(n3, rep(0, p3), Sigma3) 

beta <- (-1)^(1:p) * exp(-2 * ((1:p) - 1) / (p / 6))
z <- rnorm(n, 0, 1)
k <- 1
Beta <- array(beta, dim = c(p1, p2, p3))
Z <- array(z, dim = c(n1, n2, n3))
Ytr <- RH(X3tr, RH(X2tr, RH(X1tr, Beta))) + k * Z

######################### Arrange data  	
Y <- aperm(Ytr, c(3, 2 ,1)) 
X1 <- X3tr
X2 <- X2tr
X3 <- X1tr

######################### Construct design
if(runglmnet == 1){
	
print(c(object.size(X) / 1024^3, max(system.time(X <- kronecker(X3, kronecker(X2, X1))))))

}

######################### Time trials
#######glmnet 
if(runglmnet == 1){

glmtime[i] <- max(system.time(
fitglmnet <- glmnet(X, 
                    c(Y), 
                    family = "gaussian", 
                    standardize = FALSE, 
                    intercept = FALSE, 
                    thresh = 1e-05) ))
}

print(c("glmnet", glmtime[i]))

if(runglmnet != 1){
	
	lambda <- NULL
	nlambda <- 100
	
	}else{
		
lambda <- fitglmnet$lambda
nlambda <- length(lambda)
		
}

#######glamlasso 
glamtime[i] <-  max(system.time(
fitglamlasso <- glamlasso(list(X1, X2, X3),
                          Y, 
                          family = "gaussian",
                          penalty = "lasso",
                          nlambda = nlambda,
                          lambda.min.ratio = 0.01,
                          lambda = lambda,
                          penalty.factor = NULL,
                          reltolinner = 3 * 1e-06,
                          reltolouter = 1e-04, 
                          maxiter = 200000,
                          maxiterinner = 1000, 
                          maxiterouter = 25,
                          iwls = "exact",
                          nu = .01)))
                          
print(c("glamlasso", glamtime[i]))

######################### Get objectives	
objglm <- objective(Y, Weights = NULL,  list(X1, X2, X3), fitglmnet$beta, fitglmnet$lambda, penalty.factor = NULL, "gaussian", "lasso")
objglam <- objective(Y, Weights = NULL, list(X1, X2, X3), fitglamlasso$coef, fitglamlasso$lambda, penalty.factor = NULL, "gaussian", "lasso")

Objglm[i, ] <- c(objglm, rep(NA, 100 - length(objglm)))
Objglam[i, ] <- c(objglam, rep(NA, 100 - length(objglam)))

}

if(runglmnet == 1){

ratio <- apply((Objglam - Objglm) / abs(Objglm), 2, mean, remove.na = TRUE)
plot(ratio, type = "l", ylim = c(min(0, ratio, na.rm = TRUE), max(0, ratio, na.rm=TRUE)), col = "red", xlab = "model no.", main = "(objglamlasso - objglmnet) / abs(objglmnet) (red)")
abline(h = 0)

}
