###Script used in the simulation study in section 6.5 of Penalized Estimation in Large-Scale Generalized Linear Array Models.

### Adam Lund 2016

library(Matrix)
library(glmnet)
library(glamlasso)
library(MASS)

mult <- 1.7 #1.7 1, 1.86 2, 2.15 5, 2.31 7, 2.45 10, 2.61 14, 2.8 21
runglmnet <- 1
reps <- 5
Objglm <- Objglam  <- matrix(NA, reps, 100)
seed <- sample(1:10^6, reps) 
glmtime <- glamtime <- rep(NA, reps)

for (i in 1:reps){
	
######################### Size of example
n1 <- ceiling(20 * mult)
n2 <- ceiling(60 * mult) #biggest!!
n3 <- ceiling(10 * mult) #smallest
n <- n1 * n2 * n3
frac <- 3
p1 <- max(3, ceiling(n1 / frac))
p2 <- max(3, ceiling(n2 / frac))
p3 <- max(3, ceiling(n3 / frac))
p <- p1 * p2 * p3

######################### Simulate covariates
set.seed(seed[i])
rho <- 0.25  
sigma <- 0.51^2 

Sigma1 <- matrix(rho, p1, p1)
diag(Sigma1) <- rep(sigma, p1)
Sigma2 <- matrix(rho, p2, p2)
diag(Sigma2) <- rep(sigma, p2)
Sigma3 <- matrix(rho, p3, p3)
diag(Sigma3) <- rep(sigma, p3)

X1tr <- mvrnorm(n1, rep(0, p1), Sigma1) 
X2tr <- mvrnorm(n2, rep(0, p2), Sigma2) 
X3tr <- mvrnorm(n3, rep(0, p3), Sigma3)

beta <- (-1)^(1:p) * exp(-2 * ((1:p) - 1) / (p / 6)) * rbinom(p, 1, 0.075)
Beta <- array(beta, dim = c(p1, p2, p3))
M <- exp(RH(X3tr, RH(X2tr, RH(X1tr, Beta))))
y <- rpois(n, c(M))
Ytr <- array(y, dim = c(n1, n2, n3))
print(max(y))

######################### Arrange data  
Y <- aperm(Ytr, c(1, 2 ,3)) #123, 132, 213, 231, 312, 321
X1 <- X1tr
X2 <- X2tr
X3 <- X3tr

######################### Construct design
if(runglmnet == 1){
	
print(system.time(X <- kronecker(X3, kronecker(X2, X1))))
object.size(X) / 1024^3

}
	
######################### Time trials
#######glmnet 
if(runglmnet == 1){

glmtime[i] <- max(system.time(
fitglmnet <- glmnet(X, 
                    c(Y), 
                    family = "poisson", 
                    standardize = FALSE, 
                    intercept = FALSE, 
                    thresh = 1e-01)))

}

print(c("glmnet", glmtime[i]))

if(runglmnet != 1){
	
lambda <- NULL 
nlambda <- 100 
	
}else{
	
lambda <- fitglmnet$lambda
nlambda <- length(lambda) 
	
}

#######glamlasso 
glamtime[i] <- max(system.time(
fitglamlasso <- glamlasso(list(X1, X2, X3),
                          Y, 
                          family = "poisson",
                          penalty = "lasso",
                          nlambda = nlambda,
                          lambda.min.ratio = 1e-04,
                          lambda = lambda,
                          penalty.factor = NULL,
                          reltolinner = 4 * 1e-06, 
                          reltolouter = 1e-04, 
                          maxiter = 25000,
                          maxiterinner = 3000, 
                          maxiterouter = 25,
                          iwls = "exact",
                          nu = 0.01)))
                          
print(c("glamlasso", glamtime[i]))

######################### Get objectives
objglm <- objective(Y, Weights = NULL,  list(X1, X2, X3), fitglmnet$beta, fitglmnet$lambda, penalty.factor = NULL, "poisson", "lasso")
objglam <- objective(Y, Weights = NULL, list(X1, X2, X3), fitglamlasso$coef, fitglamlasso$lambda, penalty.factor = NULL, "poisson", "lasso")

Objglm[i, ] <- c(objglm, rep(NA, 100 - length(objglm)))
Objglam[i, ] <- c(objglam, rep(NA, 100 - length(objglam)))

}

if(runglmnet == 1){

ratio <- apply((Objglam - Objglm) / abs(Objglm), 2, mean, remove.na = TRUE)
plot(ratio, type = "l", ylim = c(min(0, ratio, na.rm = TRUE), max(0, ratio, na.rm=TRUE)), col = "red", xlab = "model no.", main = "(objglamlasso - objglmnet) / abs(objglmnet) (red)")
abline(h = 0)

}
