###Script used in the analysis of the Taxi data from section 4.3 of Penalized Estimation in Large-Scale Generalized Linear Array Models.

### Adam Lund 2016
#install.packages("~/Documents/KU/Phd/Project/Computer/StaticModel/GLAMLasso/glamlasso_2.0.tar.gz", repos = NULL, source = TRUE)
library(splancs)
library(glmnet)
library(glamlasso)

####################### load data
load("27GB/Br.rda")
load("27GB/Bc.rda")
load("27GB/Bs.rda")
load("27GB/Ytr.rda")
load(file = "27GB/Rows.rda")
load(file = "27GB/Cols.rda")
Y <- Ytr
Rows <- Rows 
Cols <- Cols 

nr <- dim(Br)[1]
nc <- dim(Bc)[1]
ns <- dim(Bs)[1]
nbr <- dim(Br)[2]
nbc <- dim(Bc)[2]
nbs <- dim(Bs)[2]

nr * nc * ns * nbr * nbc * nbs / 1024^3 * 8

image(1:dim(Ytr[ , , 1])[1], 1:dim(Ytr[ , , 1])[2], Ytr[ , , 1], col = c("black"), breaks = c(0, 1))
abline(v = le, col = "red")
abline(v = ri, col = "red")
abline(h = do, col = "red")
abline(h = up, col = "red")
points(Rows, Cols, col = "blue")
points(8, 23, col = "red")

####################### Replace data with dummy and put weights to zero
dummy <- 1000
Weights <- array(1, c(nr, nc, ns))
for(i in 1:length(Rows)){

Y[Rows[i], Cols[i], ] <- dummy
Weights[Rows[i], Cols[i], ] <- 0

}

####################### Arrange data 
Yt <- aperm(Y, c(2, 3, 1)) #medium, biggest, smallest
Weightst <- aperm(Weights, c(2, 3, 1))
Brt <- Bc
Bct <- Bs
Bst <- Br
nrt <- dim(Brt)[1]
nct <- dim(Bct)[1]
nst <- dim(Bst)[1]
nbrt <- dim(Brt)[2]
nbct <- dim(Bct)[2]
nbst <- dim(Bst)[2]

####################### Fit with glamlasso   
system.time(
fitglamlasso <- glamlasso(list(Brt, Bct, Bst),
                          Yt, 
                          family = "poisson",
                          penalty = "lasso",
                          Weights = Weightst,
                          nlambda = 100,
                          lambda.min.ratio = 1e-04,
                          lambda = NULL,
                          penalty.factor = NULL,
                          reltolinner = 1e-06, 
                          reltolouter = 1e-04, 
                          maxiter = 50000,
                          maxiterinner = 3000, 
                          maxiterouter = 25,
                          iwls = "exact",
                          nu = 0.01)
)

betaglamlasso <- fitglamlasso$coef
lambglamlasso <- fitglamlasso$lambda

####################### Compute mse
mse <- rep(NA, length(lambglamlasso))

for(i in 1: length(lambglamlasso)){##for every model

Aglamlasso <- array(betaglamlasso[, i], dim = c(nbrt, nbct, nbst))
Sglamlasso <- RH(Bst, RH(Bct, RH(Brt, Aglamlasso)))
Yest <- aperm(exp(Sglamlasso), c(3, 1, 2))

sum <- 0
for(j in 1:length(Rows)){

sum <- sum + sum((Ytr[Rows[j], Cols[j], ] - Yest[Rows[j], Cols[j], ])^2)

}

mse[i] <- sum / (length(Rows) * length(Ytr[1, 1, ]))

}

####################### Plot mse for predictions made by each model
plot(mse, type = "l")
bestmodel <- which(mse == min(mse))
abline(v = bestmodel, col = "red")

modelno <- bestmodel
Aglamlasso <- array(betaglamlasso[, modelno], dim = c(nbrt, nbct, nbst))
Sglamlasso <- RH(Bst, RH(Bct, RH(Brt, Aglamlasso)))
	
Yestbest <- aperm(exp(Sglamlasso), c(3, 1, 2))

modelno <- length(lambglamlasso)
Aglamlasso <- array(betaglamlasso[, modelno], dim = c(nbrt, nbct, nbst))
Sglamlasso <- RH(Bst, RH(Bct, RH(Brt, Aglamlasso)))

Yestlast <- aperm(exp(Sglamlasso), c(3, 1, 2))

w <- 6
h <- 4
par(mfrow = c(w, h), mai = c(0, 0, 0, 0))
for(i in 1:min(w * h, length(Rows))){

M <- max(Ytr[Rows[i], Cols[i],], Yestbest[Rows[i], Cols[i], ], Yestlast[Rows[i], Cols[i], ])
m <- min(Ytr[Rows[i], Cols[i],], Yestbest[Rows[i], Cols[i], ], Yestlast[Rows[i], Cols[i], ])

plot(Yestbest[Rows[i], Cols[i], ], col = "red", type = "l", ylim = c(m, M))
lines(Ytr[Rows[i], Cols[i], ], type = "p")
lines(Yestlast[Rows[i], Cols[i], ], col = "green")

}

